package com.iformall.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.iformall.common.IdWorker;
import com.iformall.domain.po.MallUserInfo;
import com.iformall.domain.vo.MallUserInfoVo;
import com.iformall.mapper.MallUserInfoMapper;
import com.iformall.service.MallUserInfoService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class MallUserInfoServiceImpl implements MallUserInfoService {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    MallUserInfoMapper mallUserInfoMapper;


    @Override
    public PageInfo<MallUserInfo> listAsPage(MallUserInfo record, Integer pageIndex, Integer pageSize) {
        return PageHelper.startPage(pageIndex, pageSize).doSelectPageInfo(() -> mallUserInfoMapper.findList(record));
    }

    @Override
    public int cntUserList(MallUserInfo record) {
        return mallUserInfoMapper.selectCount(new QueryWrapper<>(record));
    }

    @Override
    public MallUserInfo getById(Long id) {
        return mallUserInfoMapper.getById(id);
    }

    @Override
    public MallUserInfo getByBOpenId(String openId, String tenantId) {
        return mallUserInfoMapper.selectByBOpenId(openId, tenantId);
    }

    @Override
    public void updateBOpenId(MallUserInfo user) {
        MallUserInfo updateUser = new MallUserInfo();
        updateUser.setId(user.getId());
        updateUser.setBopenId(user.getBopenId());
        mallUserInfoMapper.updateById(user);
    }

    @Override
    public MallUserInfo getByPhone(String phone, String tenantId) {
        return mallUserInfoMapper.selectByPhone(phone, tenantId);
    }

    @Override
    public List<MallUserInfoVo> getUserByPhone(String phone) {
        return mallUserInfoMapper.selectUsersByPhone(phone);
    }

    @Override
    public List<MallUserInfo> getByWebOpenId(String webOpenId) {
        return mallUserInfoMapper.selectByWebOpenId(webOpenId);
    }

    @Override
    public List<MallUserInfoVo> getUsersByWebOpenId(String webOpenId) {
        return mallUserInfoMapper.selectUsersByWebOpenId(webOpenId);
    }

    @Override
    public void saveOrUpdate(MallUserInfo record) {
        if (record.getId() == null) {
            //record.setId(UUID.randomUUID().toString().replaceAll("-", ""));
            final IdWorker idWorker = IdWorker.get();
            record.setId(idWorker.nextId());
            mallUserInfoMapper.insert(record);
        } else {
            mallUserInfoMapper.updateById(record);
        }
    }

    @Override
    public void deleteById(Long id) {
        mallUserInfoMapper.deleteById(id);
    }

    @Override
    public MallUserInfo getByUsername(String username) {
        return mallUserInfoMapper.selectByUserName(username);
    }

    @Override
    public long cntByUserName(String username) {
        MallUserInfo userInfo = new MallUserInfo();
        userInfo.setUsername(username);
        return PageHelper.count(() -> mallUserInfoMapper.selectList(new QueryWrapper<>(userInfo)));
    }

    @Override
    public boolean hasButtonPermission(Long userId, String permission) {
        return mallUserInfoMapper.hasButtonPermission(userId, permission) == null ? false : true;
    }

    @Override
    public boolean cntByUserName(String username, Long id) {
        Map<String, Object> params = new HashMap<>();
        params.put("username", username);
        params.put("id", id);
        return mallUserInfoMapper.cntByUserName(params) > 0 ? true : false;
    }

    @Override
    public boolean cntByUserPhone(String phone, Long id) {
        Map<String, Object> params = new HashMap<>();
        params.put("phone", phone);
        params.put("id", id);
        return mallUserInfoMapper.cntByUserName(params) > 0 ? true : false;
    }

    @Override
    public List<MallUserInfo> findList(MallUserInfo record){
        return mallUserInfoMapper.findList(record);
    }


}
