package com.iformall.controller;

import com.iformall.common.Result;
import com.iformall.common.ResultData;
import com.iformall.domain.po.WxAuthorizerInfo;
import com.iformall.domain.vo.WxWeappInfo;
import com.iformall.enums.EnumAppPlat;
import com.iformall.enums.EnumWxAuthorizationStatus;
import com.iformall.service.WxAuthorizerInfoService;
import com.iformall.service.toutiao.FmTtOpenService;
import com.iformall.service.toutiao.api.bean.TtOpenAuthorizationInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import me.chanjar.weixin.common.error.WxErrorException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Date;
import java.util.List;

/**
 * Stormeye Wu
 */
@Controller
@RequestMapping("/wt_auth")
@Api(description = "微信第三方开放平台授权相关接口")
public class TtAuthController {
    private final Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    protected FmTtOpenService openService;

    @Autowired
    private WxAuthorizerInfoService authorizerInfoService;

    @ApiOperation(value = "获取授权跳转页", notes = "")
    @GetMapping("/goto_auth_url_show")
    @ResponseBody
    public String gotoPreAuthUrlShow() {
        return "<html>" +
                "<body>" +
                    "<a href='/T/wt_auth/goto_auth_url'>go</a>" +
                "</body>" +
                "</html>";
    }

    @ApiOperation(value = "授权跳转页", notes = "")
    @GetMapping("/goto_auth_url")
    public void gotoPreAuthUrl(HttpServletRequest request, HttpServletResponse response) {
        String host = request.getHeader("host");
        String url = "https://" + host + "/T/wt_auth/jump";
        try {
            url = openService.getTtOpenComponentService().getPreAuthUrl(url);
            response.sendRedirect(url);
        } catch (WxErrorException | IOException e) {
            logger.error("gotoPreAuthUrl", e);
            throw new RuntimeException(e);
        }
    }

    @ApiOperation(value = "授权", notes = "")
    @GetMapping("/jump")
    @ResponseBody
    public void jump(@RequestParam("authorization_code") String authorizationCode, @RequestParam("expires_in") Integer expiresIn) {
        try {
            TtOpenAuthorizationInfo queryAuthResult = openService.getTtOpenComponentService().getQueryAuth(authorizationCode);
            logger.info("getQueryAuth", queryAuthResult);
//            WxOpenAuthorizerInfoResult openAuthorizerInfoResult = openService.getTtOpenComponentService().getAuthorizerInfo(
//                    queryAuthResult.getAuthorizerAppid());
//            logger.info(openAuthorizerInfoResult.toString());
            // save auth info
            authorizerInfoService.saveOrUpdate(new WxAuthorizerInfo(){
                {
                    setAuthorizerAppid(queryAuthResult.getAuthorizerAppid());
                    setRefreshToken(queryAuthResult.getAuthorizerRefreshToken());
                    setAccessToken(queryAuthResult.getAuthorizerAccessToken());
                    setAccessTokenExpire(new Date(Long.valueOf(System.currentTimeMillis() + queryAuthResult.getExpiresIn()*1000)));
                    setAuthorizationStatus(EnumWxAuthorizationStatus.AUTHORIZED.getCode());
                    setAuthTime(new Date());
                }
            });
        } catch (WxErrorException e) {
            logger.error("gotoPreAuthUrl", e);
            throw new RuntimeException(e);
        }
    }

    @ApiOperation(value = "刷新", notes = "")
    @GetMapping("/retrieve")
    @ResponseBody
    public ResultData retrieve(@RequestParam("appId") String appId) {
        if(StringUtils.isNotBlank(appId)){
            try {
                retrieveAuthorizerInfo(appId);
            }catch(Exception e){
                return new ResultData(Result.ERROR, e.getMessage());
            }
        }else{
            WxWeappInfo weappInfo = new WxWeappInfo();
            weappInfo.setPlat(EnumAppPlat.TOUTIAO.getCode());
            List<WxWeappInfo> list = authorizerInfoService.getList(weappInfo);
            if(list != null && list.size() > 0){
                for (WxWeappInfo info: list) {
                    try {
                        retrieveAuthorizerInfo(info.getAuthorizerAppid());
                    }catch(Exception e){
                    }
                }
            }
        }

        return new ResultData();
    }

    private void retrieveAuthorizerInfo(String appid){
        try {
            String code = openService.getTtOpenComponentService().retrieveCode(appid);

            TtOpenAuthorizationInfo queryAuthResult = openService.getTtOpenComponentService().getQueryAuth(code);
            logger.info("getQueryAuth", queryAuthResult);
            // save auth info
            authorizerInfoService.saveOrUpdate(new WxAuthorizerInfo(){
                {
                    setAuthorizerAppid(queryAuthResult.getAuthorizerAppid());
                    setRefreshToken(queryAuthResult.getAuthorizerRefreshToken());
                    setAccessToken(queryAuthResult.getAuthorizerAccessToken());
                    setAccessTokenExpire(new Date(Long.valueOf(System.currentTimeMillis() + queryAuthResult.getExpiresIn()*1000)));
                    setAuthorizationStatus(EnumWxAuthorizationStatus.AUTHORIZED.getCode());
                    setAuthTime(new Date());
                }
            });
        } catch (WxErrorException e) {
            logger.error(appid+"找回授权码error", e);
            throw new RuntimeException(e);
        }
    }

}
